/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: affile.c,v 1.9 1999/04/08 17:50:29 bazsi Exp $
 *
 ***************************************************************************/

#include "affile.h"
#include "format.h"
#include "pkt_buffer.h"

#include <fcntl.h>
#include <assert.h>

#include "affile.c.x"

static int do_open_file(struct ol_string *name, int flags, int mode, int *fd)
{
	*fd = open(name->data, flags, mode);
	if (*fd != -1) {
		return 1;
	}
	return 0;
}

/* CLASS:
     (class 
       (name affile_source)
       (super log_source_driver)
       (vars
         (backend object io_backend)
         (src object io_fd)
         (name string)))
*/


static int do_init_affile_source(struct log_handler *c)
{
	CAST(affile_source, self, c);
	int fd;
  
	if (do_open_file(self->name, O_RDONLY, 0, &fd)) {
		
		self->src = io_read(self->backend, fd, 
			make_read_line(make_log_reader((struct log_handler *) self->super.super.next)), 
			NULL);
		return ST_OK | ST_GOON;
	}
	return ST_FAIL | ST_QUIT;
}

struct log_source_driver *make_affile_source(const char *name, struct io_backend *backend)
{
	NEW(affile_source, self);
  
	self->super.super.super.init = do_init_affile_source;
	self->backend = backend;
	self->name = c_format("%z%c", name, 0);
	self->name->length--;
	return &self->super;
}

/* CLASS:
     (class
       (name affile_dest)
       (super log_dest_driver)
       (vars
         (name string)
         (flags simple int)
         (dest object io_fd)
         (backend object io_backend)))
*/

static int do_init_affile_dest(struct log_handler *c)
{
	CAST(affile_dest, self, c);
	int fd;
	
	if (do_open_file(self->name, O_WRONLY | O_CREAT | O_APPEND, 0600, &fd)) {
		
		self->dest = io_write(self->backend, fd, 
			make_pkt_buffer(LOG_FIFO_SIZE),
			NULL);
		self->super.writer = (struct abstract_write *) self->dest->buffer;
		return ST_OK | ST_GOON;
	}
	return ST_FAIL | ST_QUIT;
}

struct log_dest_driver *make_affile_dest(const char *name, int flags, struct io_backend *backend)
{
	NEW(affile_dest, self);
	
	init_dest_driver(&self->super);
	
	self->super.super.init = do_init_affile_dest;
	self->backend = backend;
	self->name = c_format("%z%c", name, 0);
	self->name->length--;
	self->flags = flags;
	return &self->super;
}
